import { jsx as _jsx } from "react/jsx-runtime";
import { easeLinear, range, select, selectAll } from 'd3';
import { useEffect, useRef, useState } from 'react';
import { createColorGenerator, fadeAwayElements, getColor, getXScaleAndAxis, getYScaleAndAxis, } from '../utils';
import { createXAxisLabel, generateYAxisLabels, getDayData } from './utils';
export function DayOverviewHeatMap({ data, color, onTooltip, onHideTooltip, onCellClick, fade, onFadeComplete, response, fetchDayData, showXAxisLabels, ...rest }) {
    const ref = useRef(null);
    const [dayData, setDayData] = useState(data);
    useEffect(() => {
        async function fetchData() {
            const details = data.details ?? (await fetchDayData?.(data.date)) ?? [];
            setDayData((prev) => ({ ...prev, details }));
        }
        fetchData().catch((err) => {
            throw err;
        });
    }, [data.date, data.details, fetchDayData]);
    useEffect(() => {
        const margin = { top: 50, bottom: 50, left: 50, right: 50 };
        let svg;
        let resize;
        if (ref.current !== null &&
            Array.isArray(dayData.details) &&
            dayData.details?.length > 0) {
            const { dataArray, valueExtent } = getDayData(dayData);
            const binCountPerHour = Math.ceil((dayData.details.length ?? 0) / 24);
            const hourLabels = generateYAxisLabels();
            const xAxisLabels = range(0, binCountPerHour).map((e) => createXAxisLabel(e));
            const [minValue, maxValue] = valueExtent;
            const colorGenerator = createColorGenerator(minValue, maxValue, color);
            const [xScale, xAxis] = getXScaleAndAxis({
                labels: xAxisLabels,
                element: ref.current,
                margin,
                paddingInner: 0,
                response,
            });
            const [yScale, yAxis] = getYScaleAndAxis({
                labels: hourLabels,
                element: ref.current,
                margin,
                paddingInner: 0,
                response,
            });
            svg = select(ref.current)
                .append('svg')
                .attr('width', '100%')
                .attr('height', '100%')
                .style('cursor', 'pointer');
            const parent = svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`);
            if (showXAxisLabels !== false) {
                svg
                    .append('g')
                    .attr('transform', `translate(${margin.left}, ${margin.top})`)
                    .attr('class', 'x-axis')
                    .call(xAxis);
            }
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'y-axis')
                .call(yAxis);
            parent
                .selectAll('.heat-cell')
                .data(dataArray)
                .enter()
                .append('rect')
                .attr('class', 'heat-cell')
                .attr('width', xScale.bandwidth())
                .attr('height', yScale.bandwidth())
                .attr('x', (d) => xScale(createXAxisLabel(d.col)) ?? 0)
                .attr('y', (d) => yScale(hourLabels[d.hour]) ?? 0)
                .attr('fill', (d) => getColor(colorGenerator, d.value))
                .attr('stroke-width', 1)
                .attr('stroke', 'var(--background_color)');
            selectAll('.x-axis, .y-axis')
                .selectAll('text')
                .attr('fill', 'var(--primary_color)');
            selectAll('.x-axis, .y-axis')
                .selectAll('path')
                .attr('stroke', 'var(--primary_color)');
            selectAll('.heat-cell')
                .on('mouseover', (e, datum) => {
                select(e.currentTarget).attr('fill-opacity', 0.4);
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mousemove', (_e, datum) => {
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mouseout', (e) => {
                select(e.currentTarget).attr('fill-opacity', 1);
                onHideTooltip?.();
            })
                .on('click', (_e, datum) => {
                onCellClick?.(datum);
            });
            select('.x-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => createXAxisLabel(d.col) !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
            });
            select('.y-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => hourLabels[d.hour] !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
            });
            resize = () => {
                if (ref.current !== null) {
                    const [newXScale, newXAxis] = getXScaleAndAxis({
                        labels: xAxisLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0,
                        response,
                    });
                    const [newYScale, newYAxis] = getYScaleAndAxis({
                        labels: hourLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0,
                        response,
                    });
                    if (showXAxisLabels !== false) {
                        select('.x-axis')
                            .call(newXAxis)
                            .attr('transform', `translate(${margin.left}, ${margin.top})`);
                    }
                    select('.y-axis')
                        .call(newYAxis)
                        .attr('transform', `translate(${margin.left}, ${margin.top})`);
                    selectAll('.heat-cell')
                        .transition()
                        .duration(500)
                        .ease(easeLinear)
                        .attr('width', newXScale.bandwidth())
                        .attr('height', newYScale.bandwidth())
                        .attr('x', (d) => newXScale(createXAxisLabel(d.col)) ?? 0)
                        .attr('y', (d) => newYScale(hourLabels[d.hour]) ?? 0);
                }
            };
            window.addEventListener('resize', resize);
        }
        return () => {
            if (resize !== undefined) {
                window.removeEventListener('resize', resize);
            }
            svg?.remove();
        };
    }, [
        color,
        dayData,
        onCellClick,
        onHideTooltip,
        onTooltip,
        response,
        showXAxisLabels,
    ]);
    useEffect(() => {
        if (fade === true) {
            fadeAwayElements(['.heat-cell', '.x-axis', '.y-axis'])
                .then(() => {
                onFadeComplete?.();
            })
                .catch((err) => {
                throw err;
            });
        }
    }, [fade, onFadeComplete]);
    return (_jsx("div", { ...rest, style: {
            width: '100%',
            height: '100%',
            boxSizing: 'border-box',
            ...rest.style,
        }, ref: ref }));
}
//# sourceMappingURL=index.js.map