import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { maxIndex } from 'd3';
import { useCallback, useEffect, useState } from 'react';
import { DayOverviewHeatMap } from '../DayOverviewHeatmap';
import { GlobalOverviewHeatMap } from '../GlobalOverviewHeatmap';
import { MonthOverviewHeatMap } from '../MonthOverviewHeatmap';
import { fadeAwayElements } from '../utils';
import { YearOverviewHeatMap } from '../YearOverviewHeatmap';
function filterDataByYear(data, year) {
    return data.filter((ele) => new Date(ele.date).getFullYear() === year);
}
function filterMonthData(data, month) {
    return data.filter((ele) => new Date(ele.date).toLocaleString(undefined, { month: 'short' }) === month);
}
function filterSelectedDatum(data, datum) {
    return (data.find((ele) => ele.date === datum.date.toISOString()) ?? {
        date: new Date().toISOString(),
        total: NaN,
    });
}
function getBottomLabel({ overview, currentYearDate, currentMonthDate, currentDayDate, }) {
    let label = '';
    switch (overview) {
        case 'day':
            label = currentDayDate.toLocaleString(undefined, { dateStyle: 'medium' });
            break;
        case 'month':
            label = `${currentMonthDate.toLocaleDateString(undefined, {
                month: 'short',
            })} ${currentMonthDate.getFullYear()}`;
            break;
        case 'year':
            label = currentYearDate.getFullYear().toString();
            break;
        default:
            break;
    }
    return label;
}
export function DrilldownCalendar({ data, overview, color, onTooltip, onHideTooltip, response, fetchGlobalData, fetchDayData, showDayXAxisLabels, ...rest }) {
    const [overviewOrder, setOverviewOrder] = useState([overview ?? 'global']);
    const [globalData, setGlobalData] = useState([]);
    const [currentYearData, setCurrentYearData] = useState([]);
    const [currentMonthData, setCurrentMonthData] = useState([]);
    const [currentDay, setCurrentDay] = useState({
        date: new Date().toISOString(),
        total: NaN,
    });
    const [fade, setFade] = useState(false);
    useEffect(() => {
        let fetchedData = [];
        async function fetchData() {
            if (Array.isArray(data) === true && data?.length !== 0) {
                fetchedData = data ?? [];
            }
            else {
                fetchedData = (await fetchGlobalData?.()) ?? [];
            }
            setGlobalData(fetchedData);
        }
        fetchData().catch((err) => {
            throw err;
        });
    }, [data, fetchGlobalData]);
    useEffect(() => {
        if (globalData.length > 0) {
            const currentDatumIndex = maxIndex(globalData, (datum) => new Date(datum.date));
            switch (overview) {
                case 'month':
                    setCurrentMonthData(filterMonthData(globalData, new Date(globalData[currentDatumIndex].date).toLocaleString(undefined, {
                        month: 'short',
                    })));
                    break;
                case 'year':
                    setCurrentYearData(filterDataByYear(globalData, new Date(globalData[currentDatumIndex].date).getFullYear()));
                    break;
                case 'day':
                    setCurrentDay(globalData[currentDatumIndex]);
                    break;
                default:
                    break;
            }
        }
    }, [globalData, overview]);
    const onFadeComplete = useCallback(() => {
        setFade(false);
        setOverviewOrder((prev) => {
            const newOrder = [...prev];
            newOrder.pop();
            return newOrder;
        });
    }, [setFade, setOverviewOrder]);
    function getOverviewChart(input) {
        let output;
        switch (input) {
            case 'year':
                output = (_jsx(YearOverviewHeatMap, { color: color, data: currentYearData, ...rest, response: response, fade: fade, onTooltip: onTooltip, onHideTooltip: onHideTooltip, onFadeComplete: onFadeComplete, onCellClick: (d) => {
                        fadeAwayElements([
                            '.heat-cell',
                            '.x-axis',
                            '.y-axis',
                            '.month-boundary',
                        ])
                            .then(() => {
                            setCurrentDay(filterSelectedDatum(currentYearData, d));
                            setOverviewOrder((prev) => [
                                ...prev,
                                'day',
                            ]);
                        })
                            .catch((err) => {
                            throw err;
                        });
                    }, onMonthLabelClick: (d) => {
                        fadeAwayElements([
                            '.heat-cell',
                            '.x-axis',
                            '.y-axis',
                            '.month-boundary',
                        ])
                            .then(() => {
                            setCurrentMonthData(filterMonthData(currentYearData, d));
                            setOverviewOrder((prev) => [
                                ...prev,
                                'month',
                            ]);
                        })
                            .catch((err) => {
                            throw err;
                        });
                    } }));
                break;
            case 'month':
                output = (_jsx(MonthOverviewHeatMap, { color: color, data: currentMonthData, ...rest, response: response, fade: fade, onTooltip: onTooltip, onHideTooltip: onHideTooltip, onFadeComplete: onFadeComplete, onCellClick: (d) => {
                        fadeAwayElements(['.heat-cell', '.x-axis', '.y-axis'])
                            .then(() => {
                            setCurrentDay(filterSelectedDatum(currentMonthData, d));
                            setOverviewOrder((prev) => [
                                ...prev,
                                'day',
                            ]);
                        })
                            .catch((err) => {
                            throw err;
                        });
                    } }));
                break;
            case 'day':
                output = (_jsx(DayOverviewHeatMap, { color: color, data: currentDay, ...rest, showXAxisLabels: showDayXAxisLabels, fetchDayData: fetchDayData, response: response, fade: fade, onTooltip: onTooltip, onHideTooltip: onHideTooltip, onFadeComplete: onFadeComplete }));
                break;
            case 'global':
            default:
                output = (_jsx(GlobalOverviewHeatMap, { color: color, data: globalData, ...rest, response: response, fade: fade, onTooltip: onTooltip, onHideTooltip: onHideTooltip, onFadeComplete: onFadeComplete, onCellClick: (d) => {
                        fadeAwayElements(['.heat-cell', '.x-axis'])
                            .then(() => {
                            const { year } = d;
                            setCurrentYearData(filterDataByYear(globalData, year));
                            setOverviewOrder((prev) => [
                                ...prev,
                                'year',
                            ]);
                        })
                            .catch((err) => {
                            throw err;
                        });
                    } }));
                break;
        }
        return output;
    }
    return (_jsxs("div", { style: { position: 'relative' }, className: rest.className, children: [_jsx("button", { type: "button", style: {
                    position: 'absolute',
                    top: 0,
                    left: 0,
                    background: 'transparent',
                    borderColor: 'transparent',
                    zIndex: 5,
                }, disabled: overviewOrder[overviewOrder.length - 1] === 'global', onClick: () => {
                    setFade(true);
                }, children: _jsxs("svg", { xmlns: "http://www.w3.org/2000/svg", height: "24", viewBox: "0 0 24 24", width: "24", className: rest.className, children: [_jsx("path", { d: "M0 0h24v24H0z", fill: "transparent" }), _jsx("path", { d: "M11.67 3.87L9.9 2.1 0 12l9.9 9.9 1.77-1.77L3.54 12z", fill: "var(--primary_color)" })] }) }), getOverviewChart(overviewOrder[overviewOrder.length - 1]), _jsx("span", { style: {
                    position: 'absolute',
                    bottom: 0,
                    width: '100%',
                    textAlign: 'center',
                    cursor: 'pointer',
                }, children: getBottomLabel({
                    overview: overviewOrder[overviewOrder.length - 1],
                    currentYearDate: new Date(currentYearData[0]?.date ?? ''),
                    currentMonthDate: new Date(currentMonthData[0]?.date ?? ''),
                    currentDayDate: new Date(currentDay.date),
                }) })] }));
}
//# sourceMappingURL=index.js.map