import { jsx as _jsx } from "react/jsx-runtime";
import { select, selectAll, easeLinear, range } from 'd3';
import { useEffect, useRef } from 'react';
import { getXScaleAndAxis, createColorGenerator, fadeAwayElements, getColor, } from '../utils';
import { getGlobalHeatmapCellDimensions, getGlobalData } from './utils';
export function GlobalOverviewHeatMap({ data, onTooltip, onHideTooltip, color, onCellClick, fade, onFadeComplete, response, ...rest }) {
    const ref = useRef(null);
    useEffect(() => {
        const margin = { top: 50, bottom: 50, left: 50, right: 50 };
        let svg;
        let resize;
        if (ref.current !== null && data.length > 0) {
            const { dataArray, yearExtent, totalExtent } = getGlobalData(data);
            const [startYear, endYear] = yearExtent;
            const [minTotal, maxTotal] = totalExtent;
            const colorGenerator = createColorGenerator(minTotal, maxTotal, color);
            const yearLabels = range(startYear, endYear + 1).map((d) => d.toString());
            const [xScale, xAxis] = getXScaleAndAxis({
                labels: yearLabels,
                element: ref.current,
                margin,
                paddingInner: 0.2,
                response,
            });
            const [cellWidth, cellHeight] = getGlobalHeatmapCellDimensions(ref.current, xScale, margin);
            svg = select(ref.current)
                .append('svg')
                .attr('width', '100%')
                .attr('height', '100%')
                .style('cursor', 'pointer');
            const parent = svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`);
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'x-axis')
                .call(xAxis);
            parent
                .selectAll('.heat-cell')
                .data(dataArray)
                .enter()
                .append('rect')
                .attr('class', 'heat-cell')
                .attr('width', cellWidth)
                .attr('height', cellHeight)
                .attr('x', (d) => xScale(d.year.toString()) ?? 0)
                .attr('fill', (d) => getColor(colorGenerator, d.total))
                .attr('stroke-width', 1)
                .attr('stroke', ' var(--background_color)');
            select('.x-axis').selectAll('text').attr('fill', 'var(--primary_color)');
            select('.x-axis')
                .selectAll('path')
                .attr('stroke', 'var(--primary_color)');
            selectAll('.heat-cell')
                .on('mouseover', (e, datum) => {
                select(e.currentTarget).attr('fill-opacity', 0.4);
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mousemove', (_e, datum) => {
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mouseout', (e) => {
                select(e.currentTarget).attr('fill-opacity', 1);
                onHideTooltip?.();
            })
                .on('click', (_e, datum) => {
                onCellClick?.(datum);
            });
            select('.x-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => d.year.toString() !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
            });
            resize = () => {
                if (ref.current !== null) {
                    const [newXScale, newXAxis] = getXScaleAndAxis({
                        labels: yearLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0.2,
                        response,
                    });
                    const [newWidth, newHeight] = getGlobalHeatmapCellDimensions(ref.current, newXScale, margin);
                    select('.x-axis')
                        .call(newXAxis)
                        .attr('transform', `translate(${margin.left}, ${margin.top})`);
                    selectAll('.heat-cell')
                        .transition()
                        .duration(500)
                        .ease(easeLinear)
                        .attr('width', newWidth)
                        .attr('height', newHeight)
                        .attr('x', (d) => newXScale(d.year.toString()) ?? 0);
                }
            };
            window.addEventListener('resize', resize);
        }
        return () => {
            if (resize !== undefined) {
                window.removeEventListener('resize', resize);
            }
            svg?.remove();
        };
    }, [color, data, onCellClick, onHideTooltip, onTooltip, response]);
    useEffect(() => {
        if (fade === true) {
            fadeAwayElements(['.heat-cell', '.x-axis'])
                .then(() => {
                onFadeComplete?.();
            })
                .catch((err) => {
                throw err;
            });
        }
    }, [fade, onFadeComplete]);
    return (_jsx("div", { ...rest, style: {
            width: '100%',
            height: '100%',
            boxSizing: 'border-box',
            ...rest.style,
        }, ref: ref }));
}
//# sourceMappingURL=index.js.map