import { jsx as _jsx } from "react/jsx-runtime";
import { easeLinear, select, selectAll } from 'd3';
import { useEffect, useRef } from 'react';
import { createColorGenerator, getXScaleAndAxis, getYScaleAndAxis, fadeAwayElements, getColor, } from '../utils';
import { getMonthData, getWeekLabel } from './utils';
export function MonthOverviewHeatMap({ data, color, onCellClick, onTooltip, onHideTooltip, fade, onFadeComplete, response, ...rest }) {
    const ref = useRef(null);
    useEffect(() => {
        const margin = { top: 50, bottom: 50, left: 50, right: 50 };
        let svg;
        let resize;
        if (ref.current !== null && data.length > 0) {
            const { dataArray, totalExtent } = getMonthData(data);
            const dayLabels = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];
            const weekLabels = [
                ...new Set(dataArray.map((e) => getWeekLabel(e.week))),
            ];
            const [minTotal, maxTotal] = totalExtent;
            const colorGenerator = createColorGenerator(minTotal, maxTotal, color);
            const [xScale, xAxis] = getXScaleAndAxis({
                labels: weekLabels,
                element: ref.current,
                margin,
                paddingInner: 0.2,
                response,
            });
            const [yScale, yAxis] = getYScaleAndAxis({
                labels: dayLabels,
                element: ref.current,
                margin,
                paddingInner: 0,
                response,
            });
            svg = select(ref.current)
                .append('svg')
                .attr('width', '100%')
                .attr('height', '100%')
                .style('cursor', 'pointer');
            const parent = svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`);
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'x-axis')
                .call(xAxis);
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'y-axis')
                .call(yAxis);
            parent
                .selectAll('.heat-cell')
                .data(dataArray)
                .enter()
                .append('rect')
                .attr('class', 'heat-cell')
                .attr('width', xScale.bandwidth())
                .attr('height', yScale.bandwidth())
                .attr('x', (d) => xScale(getWeekLabel(d.week)) ?? 0)
                .attr('y', (d) => yScale(dayLabels[d.day - 1]) ?? 0)
                .attr('fill', (d) => getColor(colorGenerator, d.total))
                .attr('stroke-width', 1)
                .attr('pointer-events', (d) => (d.total === 0 ? 'none' : 'visiblePainted'))
                .attr('stroke', ' var(--background_color)');
            selectAll('.x-axis, .y-axis')
                .selectAll('text')
                .attr('fill', 'var(--primary_color)');
            selectAll('.x-axis, .y-axis')
                .selectAll('path')
                .attr('stroke', 'var(--primary_color)');
            selectAll('.heat-cell')
                .on('mouseover', (e, datum) => {
                select(e.currentTarget).attr('fill-opacity', 0.4);
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mousemove', (_e, datum) => {
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mouseout', (e) => {
                select(e.currentTarget).attr('fill-opacity', 1);
                onHideTooltip?.();
            })
                .on('click', (_e, datum) => {
                onCellClick?.(datum);
            });
            select('.x-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => getWeekLabel(d.week) !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
            });
            select('.y-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => dayLabels[d.day - 1] !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
            });
            resize = () => {
                if (ref.current !== null) {
                    const [newXScale, newXAxis] = getXScaleAndAxis({
                        labels: weekLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0.2,
                        response,
                    });
                    const [newYScale, newYAxis] = getYScaleAndAxis({
                        labels: dayLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0,
                        response,
                    });
                    select('.x-axis')
                        .call(newXAxis)
                        .attr('transform', `translate(${margin.left}, ${margin.top})`);
                    select('.y-axis')
                        .call(newYAxis)
                        .attr('transform', `translate(${margin.left}, ${margin.top})`);
                    selectAll('.heat-cell')
                        .transition()
                        .duration(500)
                        .ease(easeLinear)
                        .attr('width', newXScale.bandwidth())
                        .attr('height', newYScale.bandwidth())
                        .attr('x', (d) => newXScale(getWeekLabel(d.week)) ?? 0)
                        .attr('y', (d) => newYScale(dayLabels[d.day - 1]) ?? 0);
                }
            };
            window.addEventListener('resize', resize);
        }
        return () => {
            if (resize !== undefined) {
                window.removeEventListener('resize', resize);
            }
            svg?.remove();
        };
    }, [color, data, onCellClick, onHideTooltip, onTooltip, response]);
    useEffect(() => {
        if (fade === true) {
            fadeAwayElements(['.heat-cell', '.x-axis', '.y-axis'])
                .then(() => {
                onFadeComplete?.();
            })
                .catch((err) => {
                throw err;
            });
        }
    }, [fade, onFadeComplete]);
    return (_jsx("div", { ...rest, style: {
            width: '100%',
            height: '100%',
            boxSizing: 'border-box',
            ...rest.style,
        }, ref: ref }));
}
//# sourceMappingURL=index.js.map