import { jsx as _jsx } from "react/jsx-runtime";
import { easeLinear, range, select, selectAll, timeMonths } from 'd3';
import { endOfYear, startOfYear } from 'date-fns';
import { useEffect, useRef } from 'react';
import { createColorGenerator, getYScaleAndAxis, fadeAwayElements, getXScaleAndAxis, getColor, } from '../utils';
import { getYearData, getMonthBoundaryPath, getWeekPosition, fadeMonthBoundaries, brightenMonthBoundaries, getMonthScaleAndAxis, } from './utils';
export function YearOverviewHeatMap({ data, color, onTooltip, onHideTooltip, onCellClick, onMonthLabelClick, fade, onFadeComplete, response, showWeekLabels, scaleType, ...rest }) {
    const ref = useRef(null);
    useEffect(() => {
        const margin = { top: 50, bottom: 50, left: 50, right: 50 };
        let svg;
        let resize;
        if (ref.current !== null && data.length > 0) {
            const { dataArray, totalExtent } = getYearData(data);
            const [minTotal, maxTotal] = totalExtent;
            const colorGenerator = createColorGenerator(minTotal, maxTotal, color);
            const selectedDate = new Date(data[0].date);
            const monthDates = timeMonths(startOfYear(selectedDate), endOfYear(selectedDate));
            const dayLabels = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];
            const weekLabels = range(1, 54).map((e) => e.toString());
            const monthLabels = monthDates.map((e) => e.toLocaleString(undefined, { month: 'short' }));
            const firstWeekPositions = monthDates.map((d) => getWeekPosition(d));
            const [xScale, xAxis] = getXScaleAndAxis({
                labels: weekLabels,
                element: ref.current,
                margin,
                paddingInner: 0,
                response,
                align: 'bottom',
            });
            const [yScale, yAxis] = getYScaleAndAxis({
                labels: dayLabels,
                element: ref.current,
                margin,
                paddingInner: 0,
                response,
            });
            const [, monthAxis] = getMonthScaleAndAxis({
                weekPositions: firstWeekPositions,
                element: ref.current,
                margin,
                response,
                year: new Date(data[0].date).getFullYear(),
                monthLabels,
                scaleType,
            });
            svg = select(ref.current)
                .append('svg')
                .attr('width', '100%')
                .attr('height', '100%')
                .style('cursor', 'pointer');
            const parent = svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`);
            if (showWeekLabels === true) {
                svg
                    .append('g')
                    .attr('transform', `translate(${margin.left}, ${ref.current.clientHeight - margin.bottom})`)
                    .attr('class', 'x-axis week')
                    .call(xAxis);
            }
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'x-axis month')
                .call(monthAxis);
            svg
                .append('g')
                .attr('transform', `translate(${margin.left}, ${margin.top})`)
                .attr('class', 'y-axis')
                .call(yAxis);
            parent
                .selectAll('.heat-cell')
                .data(dataArray)
                .enter()
                .append('rect')
                .attr('class', 'heat-cell')
                .attr('width', xScale.bandwidth())
                .attr('height', yScale.bandwidth())
                .attr('x', (d) => xScale(weekLabels[d.week - 1]) ?? 0)
                .attr('y', (d) => yScale(dayLabels[d.day - 1]) ?? 0)
                .attr('fill', (d) => getColor(colorGenerator, d.total))
                .attr('stroke-width', 0.1)
                .attr('pointer-events', (d) => (d.total === 0 ? 'none' : 'visiblePainted'))
                .attr('stroke', 'var(--background_color)');
            const months = parent.append('g');
            months
                .selectAll('.month-boundary')
                .data(monthDates.slice(1))
                .enter()
                .append('path')
                .attr('class', 'month-boundary')
                .attr('fill', 'none')
                .attr('stroke', 'var(--background_color)')
                .attr('stroke-width', 3)
                .attr('d', (d) => getMonthBoundaryPath(d, xScale, yScale));
            selectAll('.x-axis , .y-axis')
                .selectAll('text')
                .attr('fill', 'var(--primary_color)');
            selectAll('.x-axis, .y-axis')
                .selectAll('path')
                .attr('stroke', 'var(--primary_color)');
            selectAll('.heat-cell')
                .on('mouseover', (e, datum) => {
                select(e.currentTarget).attr('fill-opacity', 0.4);
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mousemove', (_e, datum) => {
                onTooltip?.({
                    value: datum,
                });
            })
                .on('mouseout', (e) => {
                select(e.currentTarget).attr('fill-opacity', 1);
                onHideTooltip?.();
            })
                .on('click', (_e, datum) => {
                onCellClick?.(datum);
            });
            select('.y-axis')
                .selectAll('text')
                .on('mouseover', (_e, tickLabel) => {
                selectAll('.heat-cell')
                    .filter((d) => dayLabels[d.day - 1] !== tickLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
                fadeMonthBoundaries();
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
                brightenMonthBoundaries();
            });
            select('.x-axis.month')
                .selectAll('text')
                .on('mouseover', (e) => {
                const monthLabel = e.currentTarget.textContent ?? '';
                selectAll('.heat-cell')
                    .filter((d) => d.month !== monthLabel)
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 0.2);
                fadeMonthBoundaries();
            })
                .on('mouseout', () => {
                selectAll('.heat-cell')
                    .transition()
                    .duration(500)
                    .ease(easeLinear)
                    .attr('fill-opacity', 1);
                brightenMonthBoundaries();
            })
                .on('click', (e) => {
                const monthLabel = e.currentTarget.textContent ?? '';
                onMonthLabelClick?.(monthLabel);
            });
            resize = () => {
                if (ref.current !== null) {
                    const [newXScale, newXAxis] = getXScaleAndAxis({
                        labels: weekLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0,
                        response,
                        align: 'bottom',
                    });
                    const [, newMonthAxis] = getMonthScaleAndAxis({
                        weekPositions: firstWeekPositions,
                        element: ref.current,
                        margin,
                        response,
                        year: new Date(data[0].date).getFullYear(),
                        monthLabels,
                        scaleType,
                    });
                    const [newYScale, newYAxis] = getYScaleAndAxis({
                        labels: dayLabels,
                        element: ref.current,
                        margin,
                        paddingInner: 0,
                        response,
                    });
                    if (showWeekLabels === true) {
                        select('.x-axis.week')
                            .call(newXAxis)
                            .attr('transform', `translate(${margin.left}, ${ref.current.clientHeight - margin.bottom})`);
                    }
                    select('.x-axis.month').call(newMonthAxis);
                    select('.y-axis').call(newYAxis);
                    selectAll('.heat-cell')
                        .transition()
                        .duration(500)
                        .ease(easeLinear)
                        .attr('width', newXScale.bandwidth())
                        .attr('height', newYScale.bandwidth())
                        .attr('x', (d) => newXScale(weekLabels[d.week - 1]) ?? 0)
                        .attr('y', (d) => newYScale(dayLabels[d.day - 1]) ?? 0);
                    selectAll('.month-boundary')
                        .transition()
                        .duration(500)
                        .ease(easeLinear)
                        .attr('d', (d) => getMonthBoundaryPath(d, newXScale, newYScale));
                }
            };
            window.addEventListener('resize', resize);
        }
        return () => {
            if (resize !== undefined) {
                window.removeEventListener('resize', resize);
            }
            svg?.remove();
        };
    }, [
        color,
        data,
        onCellClick,
        onMonthLabelClick,
        onHideTooltip,
        onTooltip,
        response,
        showWeekLabels,
        scaleType,
    ]);
    useEffect(() => {
        if (fade === true) {
            fadeAwayElements(['.heat-cell', '.x-axis', '.y-axis', '.month-boundary'])
                .then(() => {
                onFadeComplete?.();
            })
                .catch((err) => {
                throw err;
            });
        }
    }, [fade, onFadeComplete]);
    return (_jsx("div", { ...rest, style: {
            width: '100%',
            height: '100%',
            boxSizing: 'border-box',
            ...rest.style,
        }, ref: ref }));
}
//# sourceMappingURL=index.js.map