import { axisTop } from '@d3fc/d3fc-axis';
import { easeLinear, extent, selectAll, timeDays, scaleThreshold, scaleTime, } from 'd3';
import { endOfYear, format, startOfYear } from 'date-fns';
import { removeLastYearWeekData, addAxisLabelResponsivness } from '../utils';
export function getYearData(data) {
    const dateTimeIntlFormat = new Intl.DateTimeFormat(undefined, {
        month: 'short',
    });
    let dataArray = [];
    let minTotal;
    let maxTotal;
    if (data.length > 0) {
        const dataRecord = data.reduce((acc, curr) => {
            const date = new Date(curr.date);
            return {
                ...acc,
                [`${format(date, 'I')},${format(date, 'i')},${dateTimeIntlFormat.format(date)}`]: curr,
            };
        }, {});
        const selectedDate = new Date(data[0].date);
        const minDate = startOfYear(selectedDate);
        const maxDate = endOfYear(selectedDate);
        const consecutiveDates = timeDays(minDate, maxDate);
        const consecutiveDatesRecord = consecutiveDates.reduce((acc, curr) => ({
            ...acc,
            [`${format(curr, 'I')},${format(curr, 'i')},${dateTimeIntlFormat.format(curr)}`]: {
                date: curr.toISOString(),
                total: NaN,
            },
        }), {});
        const combinedData = { ...consecutiveDatesRecord, ...dataRecord };
        dataArray = Object.entries(combinedData).map((d) => {
            const [week, day, month] = d[0].split(',');
            const { date } = d[1];
            return {
                day: Number.parseInt(day, 10),
                week: Number.parseInt(week, 10),
                month,
                total: d[1].total,
                date: new Date(date),
            };
        });
        dataArray = removeLastYearWeekData(dataArray);
        [minTotal, maxTotal] = extent(dataArray, (d) => d.total);
    }
    return {
        dataArray,
        totalExtent: [minTotal ?? NaN, maxTotal ?? NaN],
    };
}
export function getMonthBoundaryPath(date, xScale, yScale) {
    const week = Number.parseInt(format(date, 'I'), 10) - 1;
    const day = Number.parseInt(format(date, 'i'), 10) - 1;
    return `${day === 0
        ?
            `M${week * xScale.bandwidth()},0`
        : `M${(week + 1) * xScale.bandwidth()},0
         V${day * yScale.bandwidth()}
         H${week * xScale.bandwidth()}`}V${7 * yScale.bandwidth()}`;
}
export function getWeekPosition(firstDateOfMonth) {
    let weekPosition = NaN;
    const firstWeekOfMonth = Number.parseInt(format(firstDateOfMonth, 'I'), 10);
    if (firstDateOfMonth.getMonth() === 0) {
        weekPosition = 1;
    }
    else {
        weekPosition = firstWeekOfMonth;
    }
    return weekPosition;
}
export function fadeMonthBoundaries() {
    selectAll('.month-boundary')
        .transition()
        .duration(500)
        .ease(easeLinear)
        .style('opacity', 0.2);
}
export function brightenMonthBoundaries() {
    selectAll('.month-boundary')
        .transition()
        .duration(500)
        .ease(easeLinear)
        .style('opacity', 1);
}
function getMonthThresholdScaleAndAxis({ weekPositions, element, margin, monthLabels, response, }) {
    const monthScaleDomain = [...weekPositions, 53];
    let monthAxisWidth = element.clientWidth - margin.left - margin.right;
    monthAxisWidth = monthAxisWidth > 0 ? monthAxisWidth : 0;
    const monthScaleRange = monthScaleDomain.map((ele) => ((ele - 1) / 52) * monthAxisWidth);
    const monthScale = scaleThreshold()
        .domain(monthScaleDomain)
        .range([0, ...monthScaleRange, monthAxisWidth]);
    const monthAxis = axisTop(monthScale)
        .tickSize(0)
        .tickCenterLabel(true)
        .tickFormat((_, i) => monthLabels[i]);
    return [monthScale, addAxisLabelResponsivness(monthAxis, response)];
}
function getMonthTimeScaleAndAxis({ element, margin, response, year, }) {
    let monthAxisWidth = element.clientWidth - margin.left - margin.right;
    monthAxisWidth = monthAxisWidth > 0 ? monthAxisWidth : 0;
    const monthScale = scaleTime()
        .domain([new Date(year, 0, 1), new Date(year, 11, 31)])
        .range([0, monthAxisWidth]);
    const monthAxis = axisTop(monthScale)
        .tickSize(0)
        .tickFormat((d) => d.toLocaleString(undefined, { month: 'short' }));
    return [monthScale, addAxisLabelResponsivness(monthAxis, response)];
}
export function getMonthScaleAndAxis({ weekPositions, element, margin, response, year, scaleType, monthLabels, }) {
    let xScaleAndAxis;
    switch (scaleType) {
        case 'threshold':
            xScaleAndAxis = getMonthThresholdScaleAndAxis({
                weekPositions,
                element,
                margin,
                response,
                monthLabels,
            });
            break;
        case 'time':
        default:
            xScaleAndAxis = getMonthTimeScaleAndAxis({
                element,
                margin,
                response,
                year,
            });
    }
    return xScaleAndAxis;
}
//# sourceMappingURL=utils.js.map