import { axisLeft, axisTop, axisBottom, axisLabelOffset, axisLabelRotate, } from '@d3fc/d3fc-axis';
import { easeLinear, hsl, interpolateSpectral, scaleBand, scaleLinear, scaleSequential, select, selectAll, } from 'd3';
function axisLabelHide(axis) {
    const xAxisWithOffset = axisLabelOffset(axis);
    return xAxisWithOffset.decorate((container) => {
        const firstNodeTransformValue = container
            .select('text')
            .attr('transform');
        container.selectAll('text').each(function hide() {
            if (select(this).attr('transform') !== firstNodeTransformValue) {
                select(this).attr('visibility', 'hidden');
            }
        });
        return container;
    });
}
export function addAxisLabelResponsivness(axis, response) {
    let output = axis;
    switch (response) {
        case 'offset':
            output = axisLabelOffset(axis);
            break;
        case 'rotate':
            output = axisLabelRotate(axis);
            break;
        case 'hide':
        default:
            output = axisLabelHide(axis);
            break;
    }
    return output;
}
export function getXScaleAndAxis({ labels, element, margin, paddingInner, response, align, }) {
    let range = element.clientWidth - margin.left - margin.right;
    range = range > 0 ? range : 0;
    const xScale = scaleBand()
        .domain(labels)
        .range([0, range])
        .paddingInner(paddingInner);
    const axisFunction = align === 'bottom' ? axisBottom : axisTop;
    let xAxis = axisFunction(xScale).tickValues(labels).tickSize(0);
    xAxis = addAxisLabelResponsivness(xAxis, response);
    return [xScale, xAxis];
}
export function getYScaleAndAxis({ labels, element, margin, paddingInner, response, }) {
    let range = element.clientHeight - margin.top - margin.bottom;
    range = range > 0 ? range : 0;
    const yScale = scaleBand()
        .domain(labels)
        .range([0, range])
        .paddingInner(paddingInner);
    let yAxis = axisLeft(yScale).tickValues(labels).tickSize(0);
    yAxis = addAxisLabelResponsivness(yAxis, response);
    return [yScale, yAxis];
}
function getHSL(val) {
    return hsl(360 * val, 0.85, 0.7).formatHsl();
}
function getDrilldownBackgroundColor() {
    const container = document.querySelectorAll('.dark, .light');
    let initialColor = '#ffffff';
    if (container.length > 0) {
        const backgroundColor = getComputedStyle(container[0]).getPropertyValue('--background_color');
        initialColor = backgroundColor === '' ? initialColor : backgroundColor;
    }
    return initialColor;
}
export function createColorGenerator(minValue, maxValue, color) {
    let colorGenerator;
    const initialColor = getDrilldownBackgroundColor();
    switch (color) {
        case 'spectral':
            colorGenerator = scaleSequential()
                .domain([maxValue, minValue])
                .interpolator(interpolateSpectral);
            break;
        case 'hsl':
            colorGenerator = scaleSequential()
                .domain([minValue, maxValue])
                .interpolator(getHSL);
            break;
        case 'hslModified':
            colorGenerator = scaleLinear()
                .domain([minValue, (minValue + maxValue) / 2, maxValue])
                .range([
                hsl(180, 0.85, 0.6).formatHsl(),
                'white',
                hsl(0, 0.85, 0.6).formatHsl(),
            ]);
            break;
        case null:
        case undefined:
            colorGenerator = scaleLinear()
                .range([initialColor, '#ff4500'])
                .domain([minValue, maxValue]);
            break;
        default:
            colorGenerator = scaleLinear()
                .range([initialColor, color])
                .domain([minValue, maxValue]);
    }
    return colorGenerator;
}
export function removeLastYearWeekData(dataArray) {
    let updatedDataArray = [];
    if (dataArray.length > 0) {
        const firstDayOfYear = dataArray[0].day;
        updatedDataArray = dataArray;
        if (firstDayOfYear >= 5) {
            const numberOfDaysToCompleteLastYearWeek = 8 - firstDayOfYear;
            updatedDataArray = dataArray.slice(numberOfDaysToCompleteLastYearWeek);
        }
    }
    return updatedDataArray;
}
export async function fadeAwayElements(selectorsList) {
    const selectors = selectorsList.join(',');
    return selectAll(selectors)
        .transition()
        .duration(500)
        .ease(easeLinear)
        .style('opacity', 0)
        .end();
}
export function getColor(colorGenerator, value) {
    const color = Number.isFinite(value)
        ? colorGenerator(value)
        : 'var(--background_color)';
    return color;
}
//# sourceMappingURL=utils.js.map